@@ -264,9 +264,9 @@ browse_button_cb (GtkWidget *widget, gpointer data)
 						      facedir, NULL);
 	} else {
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_dialog),
-						     EXPANDED_DATADIR "/pixmaps");
+						     DATADIR "/pixmaps");
 		gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (file_dialog),
-						      EXPANDED_DATADIR "/pixmaps", NULL);
+						      DATADIR "/pixmaps", NULL);
 	}
 
 	filter = gtk_file_filter_new ();
@@ -291,11 +291,19 @@ browse_button_cb (GtkWidget *widget, gpointer data)
 	gtk_widget_show (file_dialog);
 }
 
+static void
+help_button_cb (GtkWidget *widget, gpointer data)
+{
+	g_spawn_command_line_sync ("gnome-open ghelp:gdm", NULL, NULL,
+  				   NULL, NULL);
+}
+
 int
 main (int argc, char *argv[])
 {
 	GtkWidget  *dialog;
 	GtkWidget  *browse_button;
+	GtkWidget  *help_button;
 	GtkWidget  *face_image;
 	gboolean    face_browser;
 	char	   *greeter;
@@ -307,6 +315,8 @@ main (int argc, char *argv[])
 	gtk_init(&argc, &argv);
 	photofile = g_build_filename (g_get_home_dir (), ".face", NULL);
 
+	/* Read config data in bulk */
+	gdmcomm_comm_bulk_start ();
 	face_browser = gdm_config_get_bool (GDM_KEY_BROWSER);
 	max_size     = gdm_config_get_int (GDM_KEY_USER_MAX_FILE);
 	max_width    = gdm_config_get_int (GDM_KEY_MAX_ICON_WIDTH);
@@ -314,7 +324,7 @@ main (int argc, char *argv[])
 	greeter      = gdm_config_get_string (GDM_KEY_GREETER);
 	facedir      = gdm_config_get_string (GDM_KEY_GLOBAL_FACE_DIR);
 	/* At this point we are done using the socket, so close it */
-	gdmcomm_comm_close ();
+	gdmcomm_comm_bulk_stop ();
 	imagename    = NULL;
 
 	gtk_window_set_default_icon_name ("stock_person");
@@ -324,6 +334,7 @@ main (int argc, char *argv[])
 	dialog	      = glade_xml_get_widget (xml, "face_dialog");
 	face_image    = glade_xml_get_widget (xml, "face_image");
 	browse_button = glade_xml_get_widget (xml, "browse_button");
+	help_button   = glade_xml_get_widget (xml, "helpbutton1");
 
 	gtk_widget_set_size_request (browse_button, MAX (max_width, 230), MAX (max_height, 130));
 
@@ -338,6 +349,8 @@ main (int argc, char *argv[])
 
 	g_signal_connect (browse_button, "clicked",
 			  G_CALLBACK (browse_button_cb), dialog);
+	g_signal_connect (help_button, "clicked",
+			  G_CALLBACK (help_button_cb), dialog);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (dialog_response), NULL);
