@@ -79,6 +79,8 @@ GreeterItemInfo *welcome_string_info = NULL;
 extern gboolean session_dir_whacked_out;
 extern gboolean require_quarter;
 extern gint gdm_timed_delay;
+extern GtkButton *gtk_ok_button;
+extern GtkButton *gtk_start_again_button;
 
 gboolean greeter_probably_login_prompt = FALSE;
 
@@ -190,9 +192,16 @@ process_operation (guchar       op_code,
 					gdm_config_get_string (GDM_KEY_SOUND_ON_LOGIN_FILE),
 					gdm_config_get_bool (GDM_KEY_SOUND_ON_LOGIN));
 		greeter_probably_login_prompt = TRUE;
+		if (gtk_start_again_button != NULL)
+	                gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
 	} else {
 		greeter_probably_login_prompt = FALSE;
+		if (gtk_start_again_button != NULL)
+                	gtk_widget_set_sensitive (gtk_start_again_button, TRUE);
 	}
+	if (gtk_ok_button != NULL)
+                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
+
 	greeter_ignore_buttons (FALSE);
 
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, TRUE);
@@ -207,8 +216,16 @@ process_operation (guchar       op_code,
     case GDM_NOECHO:
 	tmp = ve_locale_to_utf8 (args);
 
-	if (tmp != NULL && strcmp (tmp, _("Password:")) == 0)
+	if (tmp != NULL && strcmp (tmp, _("Password:")) == 0) {
+		if (gtk_start_again_button != NULL)
+                	gtk_widget_set_sensitive (gtk_start_again_button, TRUE);
 		greeter_probably_login_prompt = FALSE;
+	} else {
+		if (gtk_start_again_button != NULL)
+                	gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
+	}
+	if (gtk_ok_button != NULL)
+                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
 
 	greeter_ignore_buttons (FALSE);
 	greeter_item_pam_prompt (tmp, PW_ENTRY_SIZE, FALSE);
@@ -306,6 +323,11 @@ process_operation (guchar       op_code,
 
     case GDM_RESETOK:
 
+	if (gtk_ok_button != NULL)
+                gtk_widget_set_sensitive (gtk_ok_button, FALSE);
+	if (gtk_start_again_button != NULL)
+                gtk_widget_set_sensitive (gtk_start_again_button, FALSE);
+
 	conversation_info = greeter_lookup_id ("pam-conversation");
 	
 	if (conversation_info)
@@ -776,7 +798,6 @@ gdm_read_config (void)
 
 	/* Keys not to include in reread_config */
 	gdm_config_get_string (GDM_KEY_SESSION_DESKTOP_DIR);
-	gdm_config_get_string (GDM_KEY_PID_FILE);
 	gdm_config_get_string (GDM_KEY_PRE_FETCH_PROGRAM);
 
 	gdmcomm_comm_bulk_stop ();
@@ -1077,6 +1098,7 @@ main (int argc, char *argv[])
   }
   gdm_common_setup_background_color (bg_color);
   greeter_session_init ();
+  greeter_language_initialize_model ();
 
   ve_signal_add (SIGHUP, greeter_reread_config, NULL);
 
@@ -1122,20 +1144,6 @@ main (int argc, char *argv[])
 	  gdm_common_fail_greeter ("Could not set signal mask!");
   }
   
-  if G_LIKELY (! DOING_GDM_DEVELOPMENT) {
-    ctrlch = g_io_channel_unix_new (STDIN_FILENO);
-    g_io_channel_set_encoding (ctrlch, NULL, NULL);
-    g_io_channel_set_buffered (ctrlch, TRUE);
-    g_io_channel_set_flags (ctrlch, 
-			    g_io_channel_get_flags (ctrlch) | G_IO_FLAG_NONBLOCK,
-			    NULL);
-    g_io_add_watch (ctrlch, 
-		    G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
-		    (GIOFunc) greeter_ctrl_handler,
-		    NULL);
-    g_io_channel_unref (ctrlch);
-  }
-
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
   if G_UNLIKELY (DOING_GDM_DEVELOPMENT) {
@@ -1234,6 +1242,8 @@ main (int argc, char *argv[])
 			    NULL);
     }
 
+  g_free (theme_file);
+
   if G_UNLIKELY (root != NULL && greeter_lookup_id ("user-pw-entry") == NULL)
     {
       GtkWidget *dialog;
@@ -1322,6 +1332,20 @@ main (int argc, char *argv[])
   
   greeter_setup_items ();
 
+  if G_LIKELY (! DOING_GDM_DEVELOPMENT) {
+    ctrlch = g_io_channel_unix_new (STDIN_FILENO);
+    g_io_channel_set_encoding (ctrlch, NULL, NULL);
+    g_io_channel_set_buffered (ctrlch, TRUE);
+    g_io_channel_set_flags (ctrlch, 
+			    g_io_channel_get_flags (ctrlch) | G_IO_FLAG_NONBLOCK,
+			    NULL);
+    g_io_add_watch (ctrlch, 
+		    G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
+		    (GIOFunc) greeter_ctrl_handler,
+		    NULL);
+    g_io_channel_unref (ctrlch);
+  }
+
   gdm_common_setup_blinking ();
 
   gtk_widget_show_all (window);
@@ -1330,6 +1354,7 @@ main (int argc, char *argv[])
 
   greeter_item_ulist_unset_selected_user ();
   greeter_item_ulist_enable ();
+  greeter_item_ulist_check_show_userlist ();
 
   /* can it ever happen that it'd be NULL here ??? */
   if G_UNLIKELY (window->window != NULL)
@@ -1441,6 +1466,7 @@ main (int argc, char *argv[])
      gdm_config_get_string (GDM_KEY_INFO_MSG_FONT));
 
   gdm_common_setup_cursor (GDK_LEFT_PTR);
+  gdm_wm_center_cursor ();
   gdm_common_pre_fetch_launch ();
   gtk_main ();
 
